////////////////////////////////////////////////////////////////////////////////
// PAGER98                                                                    //
////////////////////////////////////////////////////////////////////////////////
// Implementation of IE4's PAGER Control                                      //
////////////////////////////////////////////////////////////////////////////////
// Version 1.00 Beta                                                          //
// Date de cration           : 20/10/1997                                    //
// Date dernire modification : 21/10/1997                                    //
////////////////////////////////////////////////////////////////////////////////
// Jean-Luc Mattei                                                            //
// jlucm@club-internet.fr  / jlucm@mygale.org                                 //
////////////////////////////////////////////////////////////////////////////////
// IMPORTANT NOTICE :                                                         //
//                                                                            //
//                                                                            //
// This program is FreeWare                                                   //
//                                                                            //
// Please do not release modified versions of this source code.               //
// If you've made any changes that you think should have been there,          //
// feel free to submit them to me at jlucm@club-internet.fr                   //
////////////////////////////////////////////////////////////////////////////////
//  REVISIONS :                                                               //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
//  NOTES :                                                                   //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

unit Pager98;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, CommCtrl, Comctl98;

type

  TCustomPager98 = class;


  TPager98ButtonState = (pbsInvisible, pbsNormal, pbsGrayed, pbsDepressed, pbsHot, pbsUnknown);
  TPager98Orientation = (poHorizontal, poVertical);

  TOnCalcSizeEvent = procedure (Sender: TCustomPager98; Var Size: Integer) of object;
  TOnScrollEvent = procedure (Sender: TCustomPager98; ScrollDir: Integer; Keys: Word; XPos, YPos: Integer; Var ScrollSize: Integer) of object;

  TCustomPager98 = class(TWinControl)
  private
    FScrollSize: Integer;
    FScrollPos: Integer;
    FPagerOrientation: TPager98Orientation;
    FDragNDrop: Boolean;
    FAutoScroll: Boolean;
    FControl: TWinControl;
    FBorderSize: Integer;
    //FBorderStyle: TPagerBorderStyle;
    FMouseForward: Boolean; // Send WM_MOUSEMOUVE to Child
    FBkColor: TColor;
    FButtonSize: Integer;
    FOnCalcSize: TOnCalcSizeEvent;
    FOnScroll: TOnScrollEvent;
    procedure CMControlChange(var Message: TCMControlChange); message CM_CONTROLCHANGE;
    procedure CNNotify(var Message: TWMNotify); message CN_NOTIFY;
    function  GetBkColor: TColor;
    function  GetBorderSize: Integer;
    function  GetButtonSize: Integer;
    function  GetLeftTopButtonState: TPager98ButtonState;
    function  GetRightBottomButtonState: TPager98ButtonState;
    function  GetScrollPosition: Integer;
    procedure SetAutoScroll(Value: Boolean);
    procedure SetBkColor(Value: TColor);
    procedure SetBorderSize(Value: Integer);
    //procedure SetBorderStyle(Value: TBorderStyle);
    procedure SetButtonSize(Value: Integer);
    procedure SetDragNDrop(Value: Boolean);
    procedure SetMouseForward(Value: Boolean);
    procedure SetPagerOrientation(Value: TPager98Orientation);
    procedure SetScrollPosition(Value: Integer);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetChild(AWinControl: TWinControl);
    procedure RecalcSize;
    procedure UpdateControl;
    property AutoScroll: Boolean read FAutoScroll write SetAutoScroll default False;
    property DragNDrop: Boolean read FDragNDrop write SetDragNDrop default False;
    property Orientation: TPager98Orientation read FPagerOrientation write SetPagerOrientation default poHorizontal;
    property ButtonSize: Integer read GetButtonSize write SetButtonSize default 12;
    property BkGndColor: TColor read GetBkColor write SetBkColor default clBtnFace;
    property BorderSize: Integer read GetBorderSize write SetBorderSize default 0;
    property ScrollPosition: Integer read GetScrollPosition write SetScrollPosition default 0;
    property ScrollSize: Integer read FScrollSize write FScrollSize default 20;
    property LeftTopButtonState: TPager98ButtonState read GetLeftTopButtonState;
    property MouseForward: Boolean read FMouseForward write SetMouseForward default False;
    property RightBottomButtonState: TPager98ButtonState read GetRightBottomButtonState;
    property OnScroll: TOnScrollEvent read FOnScroll write FOnScroll;
    property OnCalcSize: TOnCalcSizeEvent read FOnCalcSize write FOnCalcSize;
    //property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle;
  end;

  TPager98 = class(TCustomPager98)
  published
    property AutoScroll;
    property BorderSize;
    //property BorderStyle;
    property ButtonSize;
    property BkGndColor;
    //property Ctl3D;
    property DragNDrop;
    property MouseForward;
    property Orientation;
    property ScrollSize;
    property OnScroll;
    property OnCalcSize;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Exemples', [TPager98]);
end;

constructor TCustomPager98.Create(AOwner: TComponent);

begin
  inherited Create(AOwner);
  ControlStyle := [csCaptureMouse, csFixedWidth, csFixedHeight, csSetCaption, csAcceptscontrols, csClickEvents];
  Width:= 200;
  Height:= 50;
  TabStop:= True;
  FScrollSize:= 20;
  FPagerOrientation:= poHorizontal;
  FDragNDrop:= False;
  FAutoScroll:= False;
  FControl:= nil;
  FButtonSize:= 12;
  FScrollPos:= 0;
  FBorderSize:= 0;
  FBkColor:= clBtnFace;
  FOnScroll:= nil;
  FOnCalcSize:= nil;
end;

destructor TCustomPager98.Destroy;
begin
  //SetChild(Nil);
  inherited Destroy;
end;

procedure TCustomPager98.Loaded;
begin
  inherited;
  RecalcSize;
  ButtonSize:= FButtonSize;
  BkGndColor:= FBkColor;
  BorderSize:= FBorderSize;
  ScrollSize:= FScrollSize;
end;

procedure TCustomPager98.CMControlchange(var Message: TCMControlChange);
begin
  with Message do begin
    if Inserting then begin
      if (Control is TWinControl) then
        SetChild(TWincontrol(Control))
      else
        SetChild(nil);
    end
    else
      SetChild(nil);
  end;
  //inherited;
end;

procedure TCustomPager98.CreateParams(var Params: TCreateParams);
begin
  InitCommonControl(ICC_PAGESCROLLER_CLASS);
  inherited CreateParams(Params);
  CreateSubClass(Params, WC_PAGESCROLLER);
  with Params do begin
    Style := WS_VISIBLE or WS_CHILD;
    if ( FPagerOrientation = poHorizontal ) then
      Style:= Style or PGS_HORZ
    else
      Style:= Style or PGS_VERT;
    if FAutoScroll then
      Style:= Style or PGS_AUTOSCROLL;
    if FDragNDrop then
      Style:= Style or PGS_DRAGNDROP;
    {if ( FBorderStyle = bsSingle ) then
      Style:= Style or WS_BORDER;
    if ( Ctl3D ) then begin
      Style:= Style and ( Not WS_BORDER );
      ExStyle:= ExStyle or WS_EX_CLIENTEDGE;
    end;}
    WindowClass.style := WindowClass.style and not (CS_HREDRAW or CS_VREDRAW);
  end;
end;

procedure TCustomPager98.SetChild(AWinControl: TWinControl);
Var KeepSize: TSize;
begin
  FControl:= AWinControl;
  if ( Assigned(AWinControl) ) then begin
    KeepSize.cx:= FControl.Width;
    KeepSize.cy:= FControl.Height;
    Pager_SetChild(Handle, TWincontrol(FControl).Handle);
    FControl.Width:= KeepSize.cx;
    FControl.Height:= KeepSize.cy;
  end
  else
    Pager_SetChild(Handle, 0);
end;

procedure TCustomPager98.RecalcSize;
begin
  Pager_RecalcSize(Handle);
end;

procedure TCustomPager98.SetMouseForward(Value: Boolean);
begin
  if ( Value <> FMouseForward ) then begin
    FMouseForward:= Value;
    Pager_ForwardMouse(Handle, Value);
  end;
end;

function TCustomPager98.GetBkColor: TColor;
begin
  Result:= FBkColor;
end;

function TCustomPager98.GetBorderSize: Integer;
begin
  Result:= FBorderSize;
end;

function TCustomPager98.GetButtonSize: Integer;
begin
  Result:= FButtonSize;
end;

function TCustomPager98.GetLeftTopButtonState: TPager98ButtonState;
begin
  if HandleAllocated then begin
    case Pager_GetButtonState(Handle, PGB_TOPORLEFT) of
      0 : Result:= pbsInvisible;
      1 : Result:= pbsNormal;
      2 : Result:= pbsGrayed;
      4 : Result:= pbsDepressed;
      8 : Result:= pbsHot;
      else Result:= pbsUnknown;
    end;
  end
  else
    Result:= pbsUnknown;
end;

function TCustomPager98.GetRightBottomButtonState: TPager98ButtonState;
begin
  if HandleAllocated then begin
    case Pager_GetButtonState(Handle, PGB_BOTTOMORRIGHT) of
      0 : Result:= pbsInvisible;
      1 : Result:= pbsNormal;
      2 : Result:= pbsGrayed;
      4 : Result:= pbsDepressed;
      8 : Result:= pbsHot;
      else Result:= pbsUnknown;
    end;
  end
  else
    Result:= pbsUnknown;
end;

function TCustomPager98.GetScrollPosition: Integer;
begin
  Result:= Pager_GetPos(Handle);//FScrollPos;
end;

procedure TCustomPager98.SetAutoScroll(Value: Boolean);
begin
  if ( FAutoScroll <> Value ) then begin
    FAutoScroll:= Value;
    RecreateWnd;
    UpdateControl;
  end;
end;

procedure TCustomPager98.SetBkColor(Value: TColor);
begin
  if ColorToRGB(Value) <> Pager_GetBkColor(Handle) then begin
    FBkColor:= Value;
    Pager_SetBkColor(Handle, ColorToRGB(Value));
  end;
end;

procedure TCustomPager98.SetBorderSize(Value: Integer);
begin
  if Value <> Pager_GetBorder(Handle) then begin
    //FBorderSize:= Value;
    Pager_SetBorder(Handle, Value);
    FBorderSize:= Pager_GetBorder(Handle);
  end;
end;

procedure TCustomPager98.SetButtonSize(Value: Integer);
begin
  if Value <> Pager_GetButtonSize(Handle) then begin
    //FButtonSize:= Value;
    Pager_SetButtonSize(Handle, Value);
    FButtonSize:= Pager_GetButtonSize(Handle);
  end;
end;

{procedure TCustomPager98.SetBorderStyle(Value: TBorderStyle);
begin
  if ( FBorderStyle <> Value ) then begin
    FBorderStyle:= Value;
    RecreateWnd;
  end;
end;}

procedure TCustomPager98.SetDragNDrop(Value: Boolean);
begin
  if ( FDragNDrop <> Value ) then begin
    FDragNDrop:= Value;
    RecreateWnd;
    UpdateControl;
  end;
end;

procedure TCustomPager98.SetPagerOrientation(Value: TPager98Orientation);
begin
  if ( FPagerOrientation <> Value ) then begin
    FPagerOrientation:= Value;
    RecreateWnd;
    UpdateControl;
  end;
end;

procedure TCustomPager98.SetScrollPosition(Value: Integer);
Var KeepSize: TSize;
begin
  if Value <> Pager_GetPos(Handle) then begin
    //FScrollPos:= Value;
    if Assigned(FControl) then begin
      KeepSize.cx:= FControl.Width;
      KeepSize.cy:= FControl.Height;
    end;
    Pager_SetPos(Handle, Value);
    if Assigned(FControl) then begin
      FControl.Width:= KeepSize.cx;
      FControl.Height:= KeepSize.cy;
    end;
    FScrollPos:= Pager_GetPos(Handle);
  end;
end;

procedure TCustomPager98.CNNotify(var Message: TWMNotify);
begin
  with Message.NMHdr^ do begin
    case Code of
      PGN_CALCSIZE :
        begin
          with PNMPgCalcSize(Message.NMHdr)^ do begin
            case dwFlag of
              PGF_CALCWIDTH :
                begin
                  if ( FControl <> nil ) then
                    iWidth:= FControl.Width
                  else
                    iWidth:= 500;
                  if ( csDesigning in componentState ) then
                    iWidth:= iWidth+5; // Allows to view Arrow
                  if Assigned(OnCalcSize) then
                    OnCalcSize(Self, iWidth);
                end;
              PGF_CALCHEIGHT :
                begin
                  if ( FControl <> nil ) then
                    iHeight:= FControl.Height
                  else
                    iHeight:= 500;
                  if ( csDesigning in componentState ) then
                    iHeight:= iHeight+5; // Allows to view Arrow
                  if Assigned(OnCalcSize) then
                    OnCalcSize(Self, iHeight);
                end;
            end;
          end;
          Message.Result:= 0;
        end;
      PGN_SCROLL :
        begin
          with PNMPgScroll(Message.NMHdr)^ do begin
            iScroll:= FScrollSize;
            if Assigned(OnScroll) then
              OnScroll(Self, iDir, fwKeys, iXPos, iYPos, iScroll);
          end;
          Message.Result:= 0;
        end;
    end;
  end;
  //inherited;
end;

procedure TCustomPager98.UpdateControl;
begin
  SetChild(FControl);
  RecalcSize;
  ButtonSize:= FButtonSize;
  BkGndColor:= FBkColor;
  BorderSize:= FBorderSize;
end;

end.
